import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import numpy as np
import re

# 输入文件路径
file_path = "D:\\Autodock-multi\\results\\results.txt"

# 读取数据 - 增强解析能力
data = []
with open(file_path, "r", encoding="utf-8") as f:
    for line in f:
        line = line.strip()
        if not line:  # 跳过空行
            continue
            
        try:
            # 使用正则表达式匹配，更灵活地处理空格和格式问题
            import re
            match = re.match(r"([^,]+),\s*([-\d.]+)", line)
            
            if match:
                receptor_ligand = match.group(1).strip()
                affinity = float(match.group(2).strip())
                
                # 分离受体和配体名称
                parts = receptor_ligand.split("_")
                if len(parts) == 2:
                    receptor, ligand = parts
                    data.append([receptor.strip(), ligand.strip(), affinity])
                else:
                    print(f"无法正确分离受体和配体: {receptor_ligand}")
            else:
                # 尝试常规分割方法作为备选
                parts = line.split(",")
                if len(parts) >= 2:
                    receptor_ligand = parts[0].strip()
                    affinity = float(parts[1].strip())
                    
                    # 分离受体和配体名称
                    r_l_parts = receptor_ligand.split("_")
                    if len(r_l_parts) == 2:
                        receptor, ligand = r_l_parts
                        data.append([receptor.strip(), ligand.strip(), affinity])
                    else:
                        print(f"无法正确分离受体和配体: {receptor_ligand}")
                else:
                    print(f"行格式不正确: {line}")
        except Exception as e:
            print(f"处理行时出错: {line}, 错误: {e}")

# 打印数据统计信息，帮助诊断
print(f"成功解析的数据点数量: {len(data)}")
unique_receptors = set(row[0] for row in data)
unique_ligands = set(row[1] for row in data)
print(f"唯一受体数量: {len(unique_receptors)}")
print(f"唯一配体数量: {len(unique_ligands)}")
print(f"理论上应有的数据点总数: {len(unique_receptors) * len(unique_ligands)}")

# 创建DataFrame
df = pd.DataFrame(data, columns=["receptor", "ligand", "affinity"])

# 不再缩短receptor名称，保持原始名称
# 数据透视表 - 使用完整名称且确保显示所有组合
# 获取所有唯一的受体和配体
all_receptors = sorted(df['receptor'].unique())
all_ligands = sorted(df['ligand'].unique())

# 现在创建透视表，确保包含所有实际组合
df_pivot = pd.pivot_table(
    df, 
    values='affinity', 
    index='ligand', 
    columns='receptor',
    aggfunc='min'  # 如果有重复值，取最小值(亲和力越低越好)
)

# 检查透视表的覆盖情况
total_cells = len(all_receptors) * len(all_ligands)
non_null_cells = df_pivot.count().sum()
coverage = (non_null_cells / total_cells) * 100 if total_cells > 0 else 0
print(f"透视表覆盖率: {non_null_cells}/{total_cells} ({coverage:.2f}%)")

# 如果需要手动填充缺失的组合
missing_combinations = []
for receptor in all_receptors:
    for ligand in all_ligands:
        if ligand not in df_pivot.index or receptor not in df_pivot.columns or pd.isna(df_pivot.loc[ligand, receptor]):
            missing_combinations.append((receptor, ligand))

if missing_combinations:
    print(f"发现{len(missing_combinations)}个缺失的受体-配体组合")
    # 可以选择性地显示一些缺失的组合
    if len(missing_combinations) < 10:
        for receptor, ligand in missing_combinations:
            print(f"  缺失: {receptor} - {ligand}")
    else:
        for receptor, ligand in missing_combinations[:5]:
            print(f"  缺失: {receptor} - {ligand}")
        print("  ... 以及更多")

# 找出最佳和最差亲和力，用于颜色显示
vmin = df['affinity'].min()
vmax = df['affinity'].max()


# 设置可视化主题和字体
plt.style.use('seaborn-v0_8-whitegrid')
plt.rcParams.update({
    'font.family': 'Arial',
    'font.size': 10,
})

# 创建图形和绘图区域 - 增加底部边距以容纳倾斜的长标签
fig, ax = plt.subplots(figsize=(16, 10))

# 绘制热图
if df_pivot.empty:
    print("数据框为空，无法绘制热图")
else:
    # 自定义色彩映射 - 使用红色到白色到蓝色的渐变
    # 注意: 我们反转了色谱，红色现在表示较低值（更好的亲和力），蓝色表示较高值
    cmap = sns.diverging_palette(10, 240, as_cmap=True)  # 10是红色，240是蓝色
    
    # 绘制热图 - 显示所有数值，包括缺失值
    heatmap = sns.heatmap(
        df_pivot,
        cmap=cmap,
        annot=True,           # 显示数值
        fmt=".2f",            # 数值格式化为两位小数
        linewidths=0.5,       # 单元格之间的线宽
        cbar_kws={
            "label": "Affinity (kcal/mol)",
            "shrink": 0.8
        },
        vmin=vmin,            # 设置色标最小值
        vmax=vmax,            # 设置色标最大值
        annot_kws={"size": 8}, # 调整标注文本大小
        mask=False            # 不使用遮罩，显示所有数据点
    )
    
    # 旋转x轴标签为90度并倾斜放置，避免重叠
    plt.xticks(rotation=45, ha='right', fontsize=9)
    
    # 调整x轴标签位置和间距
    ax.xaxis.set_tick_params(pad=5)
    
    # 调整图表大小以适应更长的标签
    plt.subplots_adjust(bottom=0.3)
    
    plt.yticks(rotation=0, fontsize=9)
    
    # 添加标题和轴标签
    plt.title("Vina Affinity Score", fontsize=16, pad=20)
    plt.xlabel("Receptor", fontsize=12, labelpad=10)  # 修正了轴标签
    plt.ylabel("Ligand", fontsize=12, labelpad=10)    # 修正了轴标签

    
    # 调整布局以防止标签被裁剪
    plt.tight_layout()
    
    # 保存高分辨率图片
    plt.savefig("heatmap.png", dpi=300, bbox_inches='tight')
    
    # 显示图形
    plt.show()
print("热图已保存为 'heatmap.png'")